# Test pierwszości za pomocą algorytmu Rabin-Miller:
# https://www.nostarch.com/crackingcodes (na licencji BSD).

import random


def rabinMiller(num):
    # Zwraca wartość True, jeśli num to liczba pierwsza.

    s = num - 1
    t = 0
    while s % 2 == 0:
        # Podział s na pół, aż stanie się liczbą nieparzystą (i użycie
        # wartości do ustalenia liczby operacji podziału s).
        s = s // 2
        t += 1

    for trials in range(5): # Pięciokrotne sprawdzenie wartości num pod kątem pierwszości.
        a = random.randrange(2, num - 1)
        v = pow(a, s, num)
        if v != 1: # (Ta operacja nie jest przeprowadzana, jeśli wartością v jest 1.)
            i = 0
            while v != (num - 1):
                if i == t - 1:
                    return False
                else:
                    i = i + 1
                    v = (v ** 2) % num
    return True


def isPrime(num):
    # Zwraca wartość True, jeśli num to liczba pierwsza; ta funkcja pozwala na szybkie
    # sprawdzenie pierwszości liczby przed wywołaniem funkcji rabinMiller().

    if (num < 2):
        return False # 0 i 1 nie są liczbami pierwszymi.

    # W mniej więcej 1/3 można szybko sprawdzić, czy num nie jest liczbą pierwszą,
    # przez podzielenie jej przez kilkadziesiąt pierwszych liczb pierwszych, co jest szybsze
    # niż użycie funkcji rabinMiller(), choć nie wykrywa wszystkich liczb złożonych.
    lowPrimes = [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 79, 83, 89, 97, 101, 103, 107, 109, 113, 127, 131, 137, 139, 149, 151, 157, 163, 167, 173, 179, 181, 191, 193, 197, 199, 211, 223, 227, 229, 233, 239, 241, 251, 257, 263, 269, 271, 277, 281, 283, 293, 307, 311, 313, 317, 331, 337, 347, 349, 353, 359, 367, 373, 379, 383, 389, 397, 401, 409, 419, 421, 431, 433, 439, 443, 449, 457, 461, 463, 467, 479, 487, 491, 499, 503, 509, 521, 523, 541, 547, 557, 563, 569, 571, 577, 587, 593, 599, 601, 607, 613, 617, 619, 631, 641, 643, 647, 653, 659, 661, 673, 677, 683, 691, 701, 709, 719, 727, 733, 739, 743, 751, 757, 761, 769, 773, 787, 797, 809, 811, 821, 823, 827, 829, 839, 853, 857, 859, 863, 877, 881, 883, 887, 907, 911, 919, 929, 937, 941, 947, 953, 967, 971, 977, 983, 991, 997]

    if num in lowPrimes:
        return True

    # Sprawdzenie, czy którakolwiek z małych liczb dzieli się bez reszty przez num.
    for prime in lowPrimes:
        if (num % prime == 0):
            return False

    # Jeżeli inne metody zawiodą, pierwszość liczby należy sprawdzić za pomocą funkcji rabinMiller().
    return rabinMiller(num)


def generateLargePrime(keysize=1024):
    # Zwraca losowo wybraną liczbę pierwszą, której klucz ma podaną liczbę bitów.
    while True:
        num = random.randrange(2**(keysize-1), 2**(keysize))
        if isPrime(num):
            return num
